' ******************************************************************
'   Mersenne Twister Random Number Generator
' ******************************************************************
'
'   A C-program for MT19937, with initialization improved 2002/1/26. 
'   Coded by Takuji Nishimura and Makoto Matsumoto. 
'
'   Translated to FreeBASIC by Greg Lyon - April 2006 
'   Adapted for FBMath by Jean Debord - May 2006 / February 2007
'
'   Before using, initialize the state by using init_genrand(seed)
'   or init_by_array(init_key, key_length) (respectively InitMT and
'   InitMTbyArray in the FB version)
'
'   Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura, 
'   All rights reserved.
' 
'   Redistribution and use in source and binary forms, with or without 
'   modification, are permitted provided that the following conditions 
'   are met: 
' 
'     1. Redistributions of source code must retain the above copyright 
'        notice, this list of conditions and the following disclaimer. 
' 
'     2. Redistributions in binary form must reproduce the above copyright 
'        notice, this list of conditions and the following disclaimer in the 
'        documentation and/or other materials provided with the distribution. 
' 
'     3. The names of its contributors may not be used to endorse or promote 
'        products derived from this software without specific prior written 
'        permission. 
'
'   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
'   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
'   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
'   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
'   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
'   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
'   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
'   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
'   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
'   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
'   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
'
'   Any feedback is very welcome.
'   http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
'   email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
' ******************************************************************

CONST N AS UINTEGER = 624
CONST M AS UINTEGER = 397

CONST MATRIX_A   AS UINTEGER = &H9908b0dfUL  ' constant vector a
CONST UPPER_MASK AS UINTEGER = &H80000000UL  ' most significant w-r bits
CONST LOWER_MASK AS UINTEGER = &H7fffffffUL  ' least significant r bits

DIM SHARED mt(0 TO N - 1) AS UINTEGER  ' the array For the state vector 
DIM SHARED mti AS UINTEGER = N + 1     ' mti == N+1 means mt[N] is not initialized

DIM SHARED AS UINTEGER mag01(0 TO 1) = {&H0UL, MATRIX_A}

' initializes mt with a seed
SUB InitMT(Seed AS UINTEGER)

  DIM AS UINTEGER i

  mt(0)= Seed AND &HffffffffUL

  FOR i = 1 TO N-1
    mt(i) = 1812433253UL * (mt(i-1) XOR (mt(i-1) SHR 30)) + i
    ' See Knuth TAOCP Vol2. 3rd Ed. P.106 For multiplier.
    ' In the previous versions, MSBs of the seed affect
    ' only MSBs of the array mt[].
    ' 2002/01/09 modified by Makoto Matsumoto
    mt(i) = mt(i) AND &HffffffffUL
    ' For >32 Bit machines
  NEXT i

  mti = N
END SUB

' initialize by an array InitKey(0 to (key_length - 1))
SUB InitMTbyArray(InitKey() AS UINTEGER)

  DIM AS UINTEGER i, j, k, k1, key_length

  InitMT(19650218UL)

  i = 1
  j = 0

  key_length = UBOUND(InitKey) + 1
  IF N > key_length THEN k1 = N ELSE k1 = key_length
  
  FOR k = k1 TO 1 STEP -1
    mt(i) = (mt(i) Xor ((mt(i-1) Xor (mt(i-1) SHR 30)) * 1664525UL)) + InitKey(j) + j ' non linear 
    mt(i) = mt(i) AND &HffffffffUL  ' for WORDSIZE > 32 machines
    i = i + 1
    j = j + 1
    IF i >= N THEN
      mt(0) = mt(N-1)
      i = 1
    END IF
    IF j >= key_length THEN j = 0
  NEXT

  FOR k = N-1 TO 1 STEP -1
    mt(i) = (mt(i) Xor ((mt(i-1) Xor (mt(i-1) SHR 30)) * 1566083941UL)) - i ' non linear 
    mt(i) = mt(i) AND &HffffffffUL ' for WORDSIZE > 32 machines
    i = i + 1
    IF i >= N THEN
      mt(0) = mt(N-1)
      i = 1
    END IF
  NEXT

  mt(0) = &H80000000UL ' MSB is 1; assuring non-zero initial array 
END SUB

' Generates a Random number On [0,0xffffffff]-interval 
FUNCTION IRanMT() AS UINTEGER

  DIM AS UINTEGER k, y

  ' generate N words at one Time
  IF mti >= N THEN
    ' If IRanMT() has not been called, a default initial seed is used 
    IF mti = N + 1 THEN InitMT(5489UL)

    FOR k = 0 TO (N-M)-1
      y = (mt(k) AND UPPER_MASK) OR (mt(k+1) AND LOWER_MASK)
      mt(k) = mt(k+M) XOR (y SHR 1) XOR mag01(y AND &H1UL)
    NEXT

    FOR k = (N-M) TO (N-2)
      y = (mt(k) AND UPPER_MASK)OR (mt(k+1) AND LOWER_MASK)
      mt(k) = mt(k + (M-N)) XOR (y SHR 1) XOR mag01(y AND &H1UL)
    NEXT

    y = (mt(N-1) AND UPPER_MASK) OR (mt(0) AND LOWER_MASK)
    mt(N-1) = mt(M-1) XOR (y SHR 1) XOR mag01(y AND &H1UL)

    mti = 0
  END IF

  y = mt(mti)
  mti = mti + 1

  ' Tempering 
  y = y XOR (y SHR 11)
  y = y XOR ((y SHL 7) AND &H9d2c5680UL)
  y = y XOR ((y SHL 15) AND &Hefc60000UL)
  y = y XOR (y SHR 18)

  FUNCTION = y
END FUNCTION
